@@ -48,6 +48,7 @@ from stxnext.staticdeployment.interfaces import ITransformation, IDeploymentStep
     IPostTransformation, IImageTransformation
 from stxnext.staticdeployment.utils import ConfigParser, get_config_path, reset_request
 
+from stxnext.staticdeployment.app.request import fakeRequest, restoreRequest
 
 try:
     from plone.resource.file import FilesystemFile
@@ -60,6 +61,8 @@ try:
 except:
     from zope.interface import Interface as IResourceDirectory
 
+from zope.annotation import IAnnotations
+
 
 log = logging.getLogger(__name__)
 
@@ -166,7 +169,7 @@ class StaticDeploymentUtils(object):
             raise e
 
 
-    def _apply_transforms(self, html):
+    def _apply_transforms(self, html, filename=None):
         """
         Apply transforms to output html.
         """
@@ -175,7 +178,14 @@ class StaticDeploymentUtils(object):
 
         for t_name, t in transformations:
             log.debug('Processing %s transformation' % t_name)
-            html = t(html)
+            try:
+                html = t(html)
+            except:
+                if filename is None:
+                    filename = ''
+                log.error('error processing %s transformation(%s)\n%s' % (
+                    t_name, filename, traceback.format_exc()
+                    ))
         return html
 
 
@@ -214,7 +224,12 @@ class StaticDeploymentUtils(object):
         for t_name, t in transformations:
             log.debug('Processing %s image transformation for %s' % (t_name,
                 filename))
-            filename, image = t(filename, image)
+            try:
+                filename, image = t(filename, image)
+            except:
+                log.error('error in %s image-transformation(%s)\n%s' % (
+                    t_name, filename, traceback.format_exc()
+                    ))
         return filename, image
 
 
@@ -306,6 +321,35 @@ class StaticDeploymentUtils(object):
                 # call it
                 step()
 
+    def deploy_object(self, obj, context, request, section):
+        """
+        run a deploy just on one object
+        """
+        # get content for Anonymous users, not authenticated
+        noSecurityManager()
+        # assigning values
+        self.context = context
+        self.request = request
+        self.section = section
+
+        self._read_config(section)
+        self._apply_request_modifications()
+
+        # we want only objects available for anonyous users 
+        if not self._available_for_anonymous(obj):
+            return
+        # check if object is a normal page
+        is_page = obj.meta_type in self.page_types
+        try:
+            self._deploy_content(obj, is_page=is_page)
+        except:
+            log.error("error exporting object: %s\n%s" % (
+                '/'.join(obj.getPhysicalPath()),
+                traceback.format_exc())
+            )
+
+        ## find and run additional deployment steps
+        self._applay_extra_deployment_steps(None)
 
     def deploy(self, context, request, section, last_triggered=None):
         """
@@ -346,6 +390,8 @@ class StaticDeploymentUtils(object):
                          modified={'query': [modification_date, ], 'range': 'min'},
                          effectiveRange = DateTime(),
                          )
+        portal_syndication = getToolByName(self.context, 'portal_syndication')
+        site_path = '/'.join(self.context.getPhysicalPath())
         for brain in brains:
             if not brain.review_state or brain.review_state in self.deployable_review_states:
                 obj = brain.getObject()
@@ -360,6 +406,10 @@ class StaticDeploymentUtils(object):
                 is_page = brain.meta_type in self.page_types
                 try:
                     self._deploy_content(obj, is_page=is_page)
+                    if portal_syndication.isSyndicationAllowed(obj):
+                        page = '/'.join(obj.getPhysicalPath()) + '/RSS'
+                        page = page[len(site_path) + 1:]
+                        self._deploy_views([page], is_page=True)
                 except:
                     log.error("error exporting object: %s\n%s" % (
                         '/'.join(obj.getPhysicalPath()),
@@ -439,7 +489,11 @@ class StaticDeploymentUtils(object):
 
             # plone.resource file system resource
             if IResourceDirectory.providedBy(context):
-                content_obj = context[view_name]
+                try:
+                    content_obj = context[view_name]
+                except:
+                    log.warning("Unable traverse to '%s'!" % fullview_name)
+                    continue
             else:
                 content_obj = context.restrictedTraverse(view_name, None)
 
@@ -462,15 +516,16 @@ class StaticDeploymentUtils(object):
 
 
     @reset_request
-    def _render_obj(self, obj):
+    def _render_obj(self, obj, new_req=None):
         """
         Render object to string.
         """
         if isinstance(obj, basestring):
             return obj
+        if new_req is None:
+            new_req = self.request
         ## 'plone.global_sections' viewlet uses request['URL'] highlight
         ## selected tab, so it must be overridden but only for a while
-        initial_url = self.request['URL']
         try:
             obj_url = obj.absolute_url()
         except AttributeError:
@@ -479,15 +534,6 @@ class StaticDeploymentUtils(object):
             except AttributeError:
                 obj_url = None
 
-        if obj_url:
-            self.request['URL'] = obj_url
-
-        ## breadcrumb implementation in quills uses 'PARENTS', so it must
-        ## be overriden but ony for a while
-        initial_parents = self.request['PARENTS']
-        if hasattr(obj, 'aq_chain'):
-            self.request['PARENTS'] = obj.aq_chain
-
         try:
             if IResource.providedBy(obj):
                 try:
@@ -523,7 +569,7 @@ class StaticDeploymentUtils(object):
 
             if PLONE_RESOURCE_INSTALLED and isinstance(obj, FilesystemFile):
                 if not obj.request:
-                    obj.request = self.request
+                    obj.request = new_req
                     return obj().read()
 
             if PLONE_APP_BLOB_INSTALLED and IBlobWrapper.providedBy(obj):
@@ -536,9 +582,9 @@ class StaticDeploymentUtils(object):
                     return self._render_obj(def_page)
 
                 view_name = obj.getLayout()
-                view = queryMultiAdapter((obj, self.request), name=view_name)
+                view = queryMultiAdapter((obj, new_req), name=view_name)
                 if view_name == 'language-switcher':
-                    lang = self.request.get('LANGUAGE')
+                    lang = new_req.get('LANGUAGE')
                     def_page = getattr(obj, lang, None)
                     if def_page:
                         return self._render_obj(def_page)
@@ -557,8 +603,6 @@ class StaticDeploymentUtils(object):
                             return view()
                         except Exception, error:
                             log.warning("Unable to render view: '%s'! Error occurred: %s" % (view, error))
-                            pass
-
                 else:
                     try:
                         return obj()
@@ -566,12 +610,7 @@ class StaticDeploymentUtils(object):
                         pass
 
         finally:
-            ## back to initial url
-            if obj_url:
-                self.request['URL'] = initial_url
-
-            ## back to initial parents
-            self.request['PARENTS'] = initial_parents
+            pass
 
         log.warning("Not recognized object '%s'!" % repr(obj))
         return None
@@ -620,7 +659,23 @@ class StaticDeploymentUtils(object):
                     # add as already deployed resource to avoid
                     # redeployment in _deploy_resources
                     self.deployed_resources.append(file_path)
-
+        annotations = IAnnotations(obj)
+        plone_scales = annotations.get('plone.scale', {})
+        for key in plone_scales.keys():
+            info = plone_scales[key]
+            data = info['data']
+            mimetype = info['mimetype']
+            extension = mimetype.split('/')[-1]
+            file_path = "%s/%s.%s" % (
+                obj.absolute_url_path().lstrip('/'),
+                info['uid'],
+                extension)
+            content = data.open('r').read()
+            if content:
+                file_path, content = self._apply_image_transforms(file_path, content)
+                if file_path not in self.deployed_resources:
+                    self._write(file_path, content)
+                    self.deployed_resources.append(file_path)
 
     def _deploy_blob_file_field(self, obj, field):
         """
@@ -635,26 +690,7 @@ class StaticDeploymentUtils(object):
                 content = self._render_obj(str(file_instance.data))
                 if content:
                     self._write(file_path, content)
-
-
-    def _deploy_image_field(self, obj, field):
-        """
-        Deploys normal Image field
-        """
-        sizes = field.getAvailableSizes(field)
-        scalenames = sizes.keys()
-        scalenames.append(None)
-        for scalename in scalenames:
-            image =  field.getScale(obj, scale=scalename)
-            if image:
-                filename = image.getId()
-                dir_path = obj.absolute_url_path().lstrip('/')
-                file_path = os.path.join(dir_path, filename)
-                content = self._render_obj(image)
-                if content:
-                    file_path, content = self._apply_image_transforms(file_path, content)
-                    self._write(file_path, content)
-
+                    self.deployed_resources.append(file_path)
 
     def _deploy_file_field(self, obj, field):
         """
@@ -669,12 +705,17 @@ class StaticDeploymentUtils(object):
                 content = self._render_obj(str(file_instance.data))
                 if content:
                     self._write(file_path, content)
-
+                    self.deployed_resources.append(file_path)
 
     def _deploy_content(self, obj, is_page=True):
         """
         Deploy object as page.
         """
+        try:
+            new_req, orig_req = fakeRequest(obj)
+        except AttributeError:
+            # not a valid obj to override request with
+            new_req = None
         content = self._render_obj(obj)
         if content is None:
             return
@@ -709,17 +750,17 @@ class StaticDeploymentUtils(object):
             return
 
         for field in obj.Schema().fields():
-            if PLONE_APP_BLOB_INSTALLED and IBlobImageField.providedBy(field):
+            if (PLONE_APP_BLOB_INSTALLED and IBlobImageField.providedBy(field)) or \
+                    field.type == 'image':
                 self._deploy_blob_image_field(obj, field)
             elif PLONE_APP_BLOB_INSTALLED and IBlobField.providedBy(field):
                 self._deploy_blob_file_field(obj, field)
-            elif field.type == 'image':
-                self._deploy_image_field(obj, field)
             elif field.type == 'file' and obj.meta_type not in self.file_types:
                 self._deploy_file_field(obj, field)
             else:
                 continue
-
+        if new_req is not None:
+            restoreRequest(orig_req, new_req)
 
     def _deploy_resources(self, urls, base_path):
         """
@@ -774,7 +815,19 @@ class StaticDeploymentUtils(object):
                                 scalename = image_name[len(fieldname) + 1:]
                                 obj = field.getScale(parent_obj, scalename)
                                 objpath = os.path.join(objpath, 'image.jpg')
-                                continue
+                                break
+                        else:
+                            # didn't find it, just go for field name now...
+                            # could be added with archetypes.schemaextender
+                            parts = image_name.split('_')
+                            fieldname = parts[0]
+                            field = parent_obj.getField(fieldname)
+                            if len(parts) == 2:
+                                scalename = parts[1]
+                                obj = field.getScale(parent_obj, scalename)
+                                objpath = os.path.join(objpath, 'image.jpg')
+
+            add_path = True
             if not obj:
                 if '/@@images/' in objpath:
                     parent_path, image_name = objpath.split('/@@images/')
@@ -794,19 +847,37 @@ class StaticDeploymentUtils(object):
                             field = images_view.field(fieldname)
                             if field:
                                 obj = field.getScale(parent_obj, scalename)
+                            else:
+                                # need to try and get it from the uid
+                                uid, ext = fieldname.rsplit('.', 1)
+                                from plone.scale.storage import AnnotationStorage
+                                storage = AnnotationStorage(parent_obj)
+                                info = storage.get(uid)
+                                if info is not None:
+                                    obj = images_view.make(info).__of__(parent_obj)
+                                    #using the exported scale now instead
+                                    objpath = '/'.join((parent_path, fieldname))
+                                    add_path = False
                         except ComponentLookupError:
                             pass
             if not obj:
                 log.warning("Unable to deploy resource '%s'!" % objpath)
                 continue
-            if isinstance(obj, ATImage) or hasattr(obj, 'getBlobWrapper') and 'image' in obj.getBlobWrapper().getContentType():
+            if isinstance(obj, ATImage) or hasattr(obj, 'getBlobWrapper') and \
+                    'image' in obj.getBlobWrapper().getContentType() and add_path:
                 # create path to dump ATImage in original size
                 if objpath.rsplit('.', 1)[-1] in ('png', 'jpg', 'gif', 'jpeg'):
                     objpath = os.path.join(objpath, 'image.%s' % objpath.rsplit('.', 1)[-1])
                 else:
                     objpath = os.path.join(objpath, 'image.jpg')
 
-            content = self._render_obj(obj)
+            try:
+                content = self._render_obj(obj)
+            except AttributeError:
+                # XXX this can happen with CachedResource?
+                # can't figure out how but let's not error if so...
+                log.warning("Unable to deploy resource '%s'!" % objpath)
+                continue
             if content is None:
                 continue
 
@@ -851,6 +922,9 @@ class StaticDeploymentUtils(object):
         """
         filename = filename.lstrip('/')
 
+        if filename.endswith('/RSS/index.html'):
+            filename = filename.replace('/RSS/index.html', '/RSS.xml')
+
         if not content:
             log.warning("File '%s' is empty." % filename)
 
@@ -858,6 +932,9 @@ class StaticDeploymentUtils(object):
             dir_path = self.base_dir
         file_path = os.path.join(dir_path, filename)
         file_path = unquote(file_path)
+        if os.path.isfile(os.path.dirname(file_path)):
+            # in case some image resources were referenced before getting dumped
+            os.remove(os.path.dirname(file_path))
         _makedirs(os.path.dirname(file_path))
 
         try:
@@ -868,7 +945,7 @@ class StaticDeploymentUtils(object):
 
         if RE_NOT_BINARY.search(filename) and not omit_transform and \
                 not filename.endswith('.js') and not filename.endswith('.css'):
-            pre_transformated_content = self._apply_transforms(content)
+            pre_transformated_content = self._apply_transforms(content, filename)
             post_transformated_content = self._apply_post_transforms(
                     pre_transformated_content, file_path=file_path)
         else:
